package com.sun.jndi.ldap;

import javax.naming.NameClassPair;
import javax.naming.directory.BasicAttributes;
import javax.naming.spi.DirectoryManager;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import javax.naming.CompositeName;
import javax.naming.ldap.Control;
import java.util.Vector;
import javax.naming.directory.Attributes;
import javax.naming.NamingException;
import javax.naming.ldap.LdapName;
import java.security.AccessController;
import com.sun.jndi.toolkit.ctx.Continuation;
import java.security.AccessControlContext;
import javax.naming.Name;
import javax.naming.directory.SearchResult;

final class LdapSearchEnumeration extends AbstractLdapNamingEnumeration<SearchResult>
{
    private Name startName;
    private LdapCtx.SearchArgs searchArgs;
    private final AccessControlContext acc;
    
    LdapSearchEnumeration(LdapCtx ldapCtx, LdapResult ldapResult, String name, LdapCtx.SearchArgs searchArgs, Continuation continuation) throws NamingException {
        super(ldapCtx, ldapResult, searchArgs.name, continuation);
        this.searchArgs = null;
        this.acc = AccessController.getContext();
        this.startName = new LdapName(name);
        this.searchArgs = searchArgs;
    }
    
    @Override
    protected SearchResult createItem(String s, final Attributes attributes, Vector<Control> vector) throws NamingException {
        Object o = null;
        boolean isRelative = true;
        String comp;
        String string;
        try {
            LdapName ldapName = new LdapName(s);
            if (this.startName != null && ldapName.startsWith(this.startName)) {
                string = ldapName.getSuffix(this.startName.size()).toString();
                comp = ldapName.getSuffix(this.homeCtx.currentParsedDN.size()).toString();
            }
            else {
                isRelative = false;
                string = (comp = LdapURL.toUrlString(this.homeCtx.hostname, this.homeCtx.port_number, s, this.homeCtx.hasLdapsScheme));
            }
        }
        catch (NamingException ex) {
            isRelative = false;
            string = (comp = LdapURL.toUrlString(this.homeCtx.hostname, this.homeCtx.port_number, s, this.homeCtx.hasLdapsScheme));
        }
        CompositeName o2 = new CompositeName();
        if (!string.equals("")) {
            o2.add(string);
        }
        CompositeName name = new CompositeName();
        if (!comp.equals("")) {
            name.add(comp);
        }
        this.homeCtx.setParents(attributes, name);
        if (this.searchArgs.cons.getReturningObjFlag()) {
            if (attributes.get(Obj.JAVA_ATTRIBUTES[2]) != null) {
                try {
                    o = AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
                        @Override
                        public Object run() throws NamingException {
                            return Obj.decodeObject(attributes);
                        }
                    }, this.acc);
                }
                catch (PrivilegedActionException ex2) {
                    throw (NamingException)ex2.getException();
                }
            }
            if (o == null) {
                o = new LdapCtx(this.homeCtx, s);
            }
            try {
                o = DirectoryManager.getObjectInstance(o, name, isRelative ? this.homeCtx : null, this.homeCtx.envprops, attributes);
            }
            catch (NamingException ex3) {
                throw ex3;
            }
            catch (Exception rootCause) {
                NamingException o3 = new NamingException("problem generating object using object factory");
                o3.setRootCause(rootCause);
                throw o3;
            }
            String[] reqAttrs;
            if ((reqAttrs = this.searchArgs.reqAttrs) != null) {
                BasicAttributes o4 = new BasicAttributes(true);
                for (int i = 0; i < reqAttrs.length; ++i) {
                    o4.put(reqAttrs[i], null);
                }
                for (int j = 0; j < Obj.JAVA_ATTRIBUTES.length; ++j) {
                    if (o4.get(Obj.JAVA_ATTRIBUTES[j]) == null) {
                        attributes.remove(Obj.JAVA_ATTRIBUTES[j]);
                    }
                }
            }
        }
        Object o5;
        if (vector != null) {
            o5 = new SearchResultWithControls(isRelative ? o2.toString() : string, o, attributes, (boolean)isRelative, this.homeCtx.convertControls(vector));
        }
        else {
            o5 = new SearchResult(isRelative ? o2.toString() : string, o, attributes, (boolean)isRelative);
        }
        o5.setNameInNamespace(s);
        return o5;
    }
    
    @Override
    public void appendUnprocessedReferrals(LdapReferralException ex) {
        this.startName = null;
        super.appendUnprocessedReferrals(ex);
    }
    
    @Override
    protected AbstractLdapNamingEnumeration<? extends NameClassPair> getReferredResults(LdapReferralContext ldapReferralContext) throws NamingException {
        return (AbstractLdapNamingEnumeration)ldapReferralContext.search(this.searchArgs.name, this.searchArgs.filter, this.searchArgs.cons);
    }
    
    @Override
    protected void update(AbstractLdapNamingEnumeration<? extends NameClassPair> abstractLdapNamingEnumeration) {
        super.update(abstractLdapNamingEnumeration);
        this.startName = ((LdapSearchEnumeration)abstractLdapNamingEnumeration).startName;
    }
    
    void setStartName(Name startName) {
        this.startName = startName;
    }
}
